/**
 ** Copyright (c) 2005-2006 LAAS/CNRS 
 **
 ** Sara Fleury - March 2005
 **
 **
 **/

#ident "$Id: gyroClient.c,v 1.1 2006/12/02 18:33:43 sara Exp $"

#ifndef VXWORKS
#include "portLib.h"
#include <unistd.h>
#include <errno.h>
#include "h2initGlob.h"
#else
#include <vxWorks.h>
#include <tyLib.h>
#include <usrLib.h>
#include <sysLib.h>
#endif
#include <errnoLib.h>
#include <taskLib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "h2mathLib.h"
#include "../server/gyroPosterLib.h"

/*
 * GYRO_CS_CLIENT stands for GYRO Client-Server Library Client
 * It must be set if we want that gyroClient acts as a standalone external tools
 * In the other case, we will speak directly with the module
 * I assume it is smarter / easier to speak directly to the module, so the
 * option will be probably discarded soon. Please don't use it :).
 */

#ifdef GYRO_CS_CLIENT
#include "../server/gyroMsgLib.h"
#endif


/* ---------------------------------------------------------------------- */
#define GYRO_TIME_OUT 100

static BOOL gyroReplies(int res, int *bilan);
static GYRO_DATA_POSTER_STR gyroPoster;

/*
 * gyroInitAngle -------------------------------------------------------------
 *
 * Initialize gyro
 *
 * WARNING : returns TRUE or FALSE !!
 *
 * (to have same API as for KVH5000)
 */


BOOL
gyroInitAngle(char *device)
{
  if (gyroInit(device) == ERROR) return FALSE;
  if (gyroUpdateWOffset() == ERROR) return FALSE;
  if (gyroResetAngle(0.0) == ERROR) return FALSE;
  fputs("sensorsTask: started\n", stderr);
  return TRUE;
}

/*
 * gyroInit -------------------------------------------------------------
 *
 * Initialize gyro
 */


#ifdef GYRO_CS_CLIENT
static GYRO_CS_CLIENT_ID clientId=NULL;
#endif

STATUS
gyroInit(char *device)
{
  double theta;

#ifdef GYRO_CS_CLIENT
  char *mboxName = "gyroClient";      /* mbox name */
  int activity, bilan, res;

  /* already connected */
  if (clientId != NULL) {
    return OK;
  }
#endif

  /* find out poster */
  if (gyroDataPosterInit() == ERROR) {
    printf ("gyroInit: gyroDataPosterInit failed\n");
    return ERROR;
  }

#ifdef GYRO_CS_CLIENT
  /* Initialisation des boites aux lettres */
  if (csMboxInit (mboxName, 0, GYRO_GYRO_CS_CLIENT_MBOX_REPLY_SIZE) != OK) {
    h2perror("gyroInit: csMboxInit failed\n");
    return (ERROR);
  }

  /* S'initialiser comme client */
  if (gyroClientInit(&clientId) != OK) {
      h2perror("gyroInit: gyroClientInit failed\n");
      csMboxEnd();
      clientId = NULL;
      return (ERROR);
    }

  /* send the init request */
  res = gyroConnectRqstAndRcv(clientId, GYRO_TIME_OUT, device, &activity, &bilan);
  if (!gyroReplies(res, &bilan)) 
    return ERROR;
#else
  printf ("gyroInit: requests off -> ask directly to the module\n");
#endif

  /* try to read once */
  if (gyroReadCorrAngle(&theta) == FALSE) {
    printf ("gyroInit: cannot read poster\n");
    return ERROR;
  }

  return OK;
}


/*
 * gyroEnd  -------------------------------------------------------------
 *
 * End connection to serial port
 *
 * Returns : WARNING : BOOL :  FALSE if failed or TRUE.
 */

BOOL gyroEnd(void)
{
#ifdef GYRO_CS_CLIENT
  int res, activity, bilan;
  res = gyroDisconnectRqstAndRcv(clientId, GYRO_TIME_OUT, &activity, &bilan);
  return gyroReplies(res, &bilan);
#else
  return TRUE;
#endif
}

/*
 * gyroStop -------------------------------------------------------------
 *
 * stop the driver (all the module !)
 */

STATUS
gyroStop(void)
{
#ifdef GYRO_CS_CLIENT
  if (clientId==NULL) {
    return OK;
  }
  gyroClientEnd(clientId);
  csMboxEnd();
  clientId = NULL;
#endif
  return OK;
}

/*
 * gyroUpdateWOffset -----------------------------------------------------
 *
 * Computes the w offset (theoritically depends on temperature)
 * Controls also the period
 *
 *         WARNING: the robot must be stopped
 *
 * It returns when the computation is over
 */

#define GYRO_MEASURE_OFFSET_DURATION 30 /* seconds */

STATUS
gyroUpdateWOffset(void)
{
#ifdef GYRO_CS_CLIENT
  int activity, bilan, res;
  res = gyroCalibrateOffsetRqstAndRcv(clientId, 0, &activity, &bilan);
  if (!gyroReplies(res, &bilan))
    return ERROR;
#else
  printf ("gyroUpdateWOffset: requests off -> ask directly to the module !\n");
#endif
  return OK;
}

/*
 * gyroResetAngle -------------------------------------------------------------
 *
 * Reset the angle value
 */

STATUS
gyroResetAngle(double angle)
{
#ifdef GYRO_CS_CLIENT
  int bilan, res;
  res = gyroSetAngleRqstAndRcv(clientId, &angle, &bilan);
  if (!gyroReplies(res, &bilan))
    return ERROR;
#else
  printf ("gyroResetAngle: requests off -> ask directly to the module !\n");
#endif
  return OK;
}


/*
 * gyroPeriod -----------------------------------------------------------
 *
 * returns the period of the gyro
 */

STATUS
gyroPeriod(double *period)
{
   printf ("gyroPeriod: not implemented yet\n");
   return OK;
}


/*
 * gyroShow -------------------------------------------------------------
 *
 * print gyro data
 */

void
gyroShow(void)
{
  if (gyroDataPosterRead(&gyroPoster) == ERROR) {
    printf ("gyroRate: gyroDataPosterRead failed\n");
    return;
  }

#if 0
   printf("Gyro:\n\n");
   printf("Device          : %s\n", str.device);
   printf("Read task       : 0x%x\n", str.taskId);

   if (str.snO)
      printf("Serial number   : %d\n", str.sn);
   else
      printf("Serial number   : N/A\n");
#endif

   printf("Angle           : %.2f deg\n", gyroPoster.angle.yaw);

   printf("\n");
}

/*
 * gyroReadAngle ---------------------------------------------------------
 *
 * WARNING : return TRUE or FALSE 
 */

BOOL gyroReadAngle(double *theta)
{
  fprintf(stderr, "gyroReadAngle is not implemented anymore, used gyroReadCorrAngle\n");
  return FALSE;
}

/*
 * gyroReadCorrAngle ---------------------------------------------------------
 *
 * Like gyroReadAngle but correct the resulting 
 *                         angle taking into account the earth rotation from 
 *                         the gyroInit().
 *
 * WARNING 1 : to be use only if the gyro is horizontal !
 *
 * WARNING 2 : return TRUE or FALSE 
 * (to have same API as for KVH5000)
 */

BOOL gyroReadCorrAngle(double *theta)
{
  if (gyroDataPosterRead(&gyroPoster) == ERROR) {
    printf ("gyroReadCorrAngle: gyroDataPosterRead failed\n");
    return FALSE;
  }
  *theta = gyroPoster.angle.yaw;
  return TRUE;
}



static BOOL gyroReplies(int res, int *bilan)
{
  /* test  result */
  switch(res) {
    case FINAL_REPLY_OK:
      printf("-- Final reply: ");
      break;

    case FINAL_REPLY_TIMEOUT:
      printf("** Final reply time out: \n");
      break;

    case ERROR:
      if (H2_MODULE_ERR_FLAG(*bilan))
	printf("-- Final reply: ");
      else 
	printf("** Error : ");
      break;

    default:
      printf("** Error (status %d) ", res);
      break;
    } /* switch */
  *bilan = errnoGet();
  h2printErrno(*bilan);

  if (*bilan == OK)
    return TRUE;
  else
    return FALSE;
}
